@@ -1411,8 +1411,8 @@ class PPField(six.with_metaclass(abc.ABCMeta, object)):
                     header_elem = int(header_elem)
                 lb[index] = header_elem
             else:
-                index = slice(pos[0] - NUM_LONG_HEADERS, pos[-1]
-                              - NUM_LONG_HEADERS + 1)
+                index = slice(pos[0] - NUM_LONG_HEADERS,
+                              pos[-1] - NUM_LONG_HEADERS + 1)
                 b[index] = header_elem
 
         # Although all of the elements are now populated, we still need to
@@ -2442,8 +2442,8 @@ def as_fields(cube, field_coords=None, target=None):
         A filename or open file handle.
 
     """
-    return (field for cube, field in as_pairs(cube, field_coords=field_coords,
-                                              target=target))
+    return (field for cube, field in save_pairs_from_cube(
+        cube, field_coords=field_coords, target=target))
 
 
 def save_fields(fields, target, append=False):
