@@ -34,6 +34,7 @@ from __future__ import (absolute_import, division, print_function)
 from six.moves import (filter, input, map, range, zip)  # noqa
 import six
 
+import codecs
 import collections
 import contextlib
 import difflib
@@ -41,14 +42,17 @@ import filecmp
 import functools
 import gzip
 import inspect
+import json
 import io
 import logging
+import math
 import os
 import os.path
 import shutil
 import subprocess
 import sys
 import unittest
+import threading
 import warnings
 import xml.dom.minidom
 import zlib
@@ -58,8 +62,10 @@ try:
 except ImportError:
     import mock
 
+import filelock
 import numpy as np
 import numpy.ma as ma
+import requests
 
 import iris.cube
 import iris.config
@@ -109,9 +115,18 @@ try:
 except ImportError:
     NC_TIME_AXIS_AVAILABLE = False
 
+try:
+    requests.get('https://github.com/SciTools/iris')
+    INET_AVAILABLE = True
+except requests.exceptions.ConnectionError:
+    INET_AVAILABLE = False
 
 #: Basepath for test results.
 _RESULT_PATH = os.path.join(os.path.dirname(__file__), 'results')
+#: Default perceptual hash size.
+_HASH_SIZE = 16
+#: Default maximum perceptual hash hamming distance.
+_HAMMING_DISTANCE = 2
 
 if '--data-files-used' in sys.argv:
     sys.argv.remove('--data-files-used')
@@ -139,7 +154,8 @@ if (MPL_AVAILABLE and '-d' in sys.argv):
     plt.switch_backend('tkagg')
     _DISPLAY_FIGURES = True
 
-_DEFAULT_IMAGE_TOLERANCE = 10.0
+# Threading non re-entrant blocking lock to ensure thread-safe plotting.
+_lock = threading.Lock()
 
 
 def main():
@@ -174,7 +190,10 @@ def get_data_path(relative_path):
     """
     if not isinstance(relative_path, six.string_types):
         relative_path = os.path.join(*relative_path)
-    data_path = os.path.join(iris.config.TEST_DATA_DIR, relative_path)
+    test_data_dir = iris.config.TEST_DATA_DIR
+    if test_data_dir is None:
+        test_data_dir = ''
+    data_path = os.path.join(test_data_dir, relative_path)
 
     if _EXPORT_DATAPATHS_FILE is not None:
         _EXPORT_DATAPATHS_FILE.write(data_path + '\n')
@@ -262,8 +281,7 @@ class IrisTest(unittest.TestCase):
                               filename)
         return result
 
-    def assertCMLApproxData(self, cubes, reference_filename=None, *args,
-                            **kwargs):
+    def assertCMLApproxData(self, cubes, reference_filename=None, **kwargs):
         # passes args and kwargs on to approx equal
         if isinstance(cubes, iris.cube.Cube):
             cubes = [cubes]
@@ -272,12 +290,11 @@ class IrisTest(unittest.TestCase):
             reference_filename = [self.get_result_path(reference_filename)]
         for i, cube in enumerate(cubes):
             fname = list(reference_filename)
-            # don't want the ".cml" for the numpy data file
+            # don't want the ".cml" for the json stats file
             if fname[-1].endswith(".cml"):
                 fname[-1] = fname[-1][:-4]
-            fname[-1] += '.data.%d.npy' % i
-            self.assertCubeDataAlmostEqual(cube, fname, *args, **kwargs)
-
+            fname[-1] += '.data.%d.json' % i
+            self.assertCubeDataAlmostEqual(cube, fname, **kwargs)
         self.assertCML(cubes, reference_filename, checksum=False)
 
     def assertCDL(self, netcdf_filename, reference_filename=None, flags='-h'):
@@ -389,32 +406,38 @@ class IrisTest(unittest.TestCase):
             diff = ''.join(difflib.unified_diff(reference_text, source_text, 'Reference', 'Test result', '', '', 0))
             self.fail("%s does not match reference file: %s\n%s" % (desc, reference_filename, diff))
 
-    def assertCubeDataAlmostEqual(self, cube, reference_filename, *args, **kwargs):
+    def assertCubeDataAlmostEqual(self, cube, reference_filename, **kwargs):
         reference_path = self.get_result_path(reference_filename)
         if self._check_reference_file(reference_path):
             kwargs.setdefault('err_msg', 'Reference file %s' % reference_path)
-
-            result = np.load(reference_path)
-            if isinstance(result, np.lib.npyio.NpzFile):
-                self.assertIsInstance(cube.data, ma.MaskedArray, 'Cube data was not a masked array.')
-                # Avoid comparing any non-initialised array data.
-                data = cube.data.filled()
-                np.testing.assert_array_almost_equal(data, result['data'],
-                                                     *args, **kwargs)
-                np.testing.assert_array_equal(cube.data.mask, result['mask'])
-            else:
-                np.testing.assert_array_almost_equal(cube.data, result, *args, **kwargs)
+            with open(reference_path, 'r') as reference_file:
+                stats = json.load(reference_file)
+                self.assertEqual(stats.get('shape', []), list(cube.shape))
+                self.assertEqual(stats.get('masked', False),
+                                       isinstance(cube.data, ma.MaskedArray))
+                nstats = np.array((stats.get('mean', 0.), stats.get('std', 0.),
+                                   stats.get('max', 0.), stats.get('min', 0.)),
+                                  dtype=np.float_)
+                if math.isnan(stats.get('mean', 0.)):
+                    self.assertTrue(math.isnan(cube.data.mean()))
+                else:
+                    cube_stats = np.array((cube.data.mean(), cube.data.std(),
+                                           cube.data.max(), cube.data.min()),
+                                          dtype=np.float_)
+                    self.assertArrayAllClose(nstats, cube_stats, **kwargs)
         else:
             self._ensure_folder(reference_path)
             logger.warning('Creating result file: %s', reference_path)
+            masked = False
             if isinstance(cube.data, ma.MaskedArray):
-                # Avoid recording any non-initialised array data.
-                data = cube.data.filled()
-                with open(reference_path, 'wb') as reference_file:
-                    np.savez(reference_file, data=data, mask=cube.data.mask)
-            else:
-                with open(reference_path, 'wb') as reference_file:
-                    np.save(reference_file, cube.data)
+                masked = True
+            stats = {'mean': np.float_(cube.data.mean()),
+                     'std': np.float_(cube.data.std()),
+                     'max': np.float_(cube.data.max()),
+                     'min': np.float_(cube.data.min()),
+                     'shape': cube.shape, 'masked': masked}
+            with open(reference_path, 'w') as reference_file:
+                reference_file.write(json.dumps(stats))
 
     def assertFilesEqual(self, test_filename, reference_filename):
         reference_path = self.get_result_path(reference_filename)
@@ -620,16 +643,21 @@ class IrisTest(unittest.TestCase):
 
         """
         # Obtain a consistent ID for the current test.
-
         # NB. unittest.TestCase.id() returns different values depending on
         # whether the test has been run explicitly, or via test discovery.
         # For example:
         #   python tests/test_plot.py => '__main__.TestContourf.test_tx'
         #   ird -t => 'iris.tests.test_plot.TestContourf.test_tx'
-        bits = self.id().split('.')[-3:]
+        bits = self.id().split('.')
         if bits[0] == '__main__':
-            file_name = os.path.basename(sys.modules['__main__'].__file__)
+            floc = sys.modules['__main__'].__file__
+            path, file_name = os.path.split(os.path.abspath(floc))
             bits[0] = os.path.splitext(file_name)[0]
+            folder, location = os.path.split(path)
+            bits = [location] + bits
+            while location not in ['iris', 'example_tests']:
+                folder, location = os.path.split(folder)
+                bits = [location] + bits
         test_id = '.'.join(bits)
 
         # Derive the sequential assertion ID within the test
@@ -652,21 +680,27 @@ class IrisTest(unittest.TestCase):
             logger.warning('Creating folder: %s', dir_path)
             os.makedirs(dir_path)
 
-    def check_graphic(self, tol=_DEFAULT_IMAGE_TOLERANCE):
-        """Checks the CRC matches for the current matplotlib.pyplot figure, and closes the figure."""
+    def check_graphic(self):
+        """
+        Check the hash of the current matplotlib figure matches the expected
+        image hash for the current graphic test.
 
+        To create missing image test results, set the IRIS_TEST_CREATE_MISSING
+        environment variable before running the tests. This will result in new
+        and appropriately "<hash>.png" image files being generated in the image
+        output directory, and the imagerepo.json file being updated.
+
+        """
+        import imagehash
+        from PIL import Image
+
+        dev_mode = os.environ.get('IRIS_TEST_CREATE_MISSING')
         unique_id = self._unique_id()
-
-        figure = plt.gcf()
+        repo_fname = os.path.join(_RESULT_PATH, 'imagerepo.json')
+        with open(repo_fname, 'rb') as fi:
+            repo = json.load(codecs.getreader('utf-8')(fi))
 
         try:
-            expected_fname = os.path.join(os.path.dirname(__file__),
-                                          'results', 'visual_tests',
-                                          unique_id + '.png')
-
-            if not os.path.isdir(os.path.dirname(expected_fname)):
-                os.makedirs(os.path.dirname(expected_fname))
-
             #: The path where the images generated by the tests should go.
             image_output_directory = os.path.join(os.path.dirname(__file__),
                                                   'result_image_comparison')
@@ -682,31 +716,81 @@ class IrisTest(unittest.TestCase):
             result_fname = os.path.join(image_output_directory,
                                         'result-' + unique_id + '.png')
 
-            if not os.path.isdir(os.path.dirname(result_fname)):
+            if not os.path.isdir(image_output_directory):
                 # Handle race-condition where the directories are
                 # created sometime between the check above and the
                 # creation attempt below.
                 try:
-                    os.makedirs(os.path.dirname(result_fname))
+                    os.makedirs(image_output_directory)
                 except OSError as err:
                     # Don't care about "File exists"
                     if err.errno != 17:
                         raise
 
-            figure.savefig(result_fname)
+            def _create_missing():
+                fname = '{}.png'.format(phash)
+                base_uri = ('https://scitools.github.io/test-iris-imagehash/'
+                            'images/{}')
+                uri = base_uri.format(fname)
+                hash_fname = os.path.join(image_output_directory, fname)
+                uris = repo.setdefault(unique_id, [])
+                uris.append(uri)
+                print('Creating image file: {}'.format(hash_fname))
+                figure.savefig(hash_fname)
+                msg = 'Creating imagerepo entry: {} -> {}'
+                print(msg.format(unique_id, uri))
+                lock = filelock.FileLock(os.path.join(_RESULT_PATH,
+                                                      'imagerepo.lock'))
+                # The imagerepo.json file is a critical resource, so ensure
+                # thread safe read/write behaviour via platform independent
+                # file locking.
+                with lock.acquire(timeout=600):
+                    with open(repo_fname, 'wb') as fo:
+                        json.dump(repo, codecs.getwriter('utf-8')(fo),
+                                  indent=4, sort_keys=True)
 
-            if not os.path.exists(expected_fname):
-                warnings.warn('Created image for test %s' % unique_id)
-                shutil.copy2(result_fname, expected_fname)
+            # Calculate the test result perceptual image hash.
+            buffer = io.BytesIO()
+            figure = plt.gcf()
+            figure.savefig(buffer, format='png')
+            buffer.seek(0)
+            phash = imagehash.phash(Image.open(buffer), hash_size=_HASH_SIZE)
 
-            err = mcompare.compare_images(expected_fname, result_fname, tol=tol)
+            if unique_id not in repo:
+                if dev_mode:
+                    _create_missing()
+                else:
+                    figure.savefig(result_fname)
+                    emsg = 'Missing image test result: {}.'
+                    raise ValueError(emsg.format(unique_id))
+            else:
+                uris = repo[unique_id]
+                # Create the expected perceptual image hashes from the uris.
+                to_hash = imagehash.hex_to_hash
+                expected = [to_hash(os.path.splitext(os.path.basename(uri))[0],
+                                    hash_size=_HASH_SIZE)
+                            for uri in uris]
+
+                # Calculate the hamming distance vector for the result hash.
+                distances = [e - phash for e in expected]
+
+                if np.all([hd > _HAMMING_DISTANCE for hd in distances]):
+                    if dev_mode:
+                        _create_missing()
+                    else:
+                        figure.savefig(result_fname)
+                        msg = ('Bad phash {} with hamming distance {} '
+                               'for test {}.')
+                        msg = msg.format(phash, distances, unique_id)
+                        if _DISPLAY_FIGURES:
+                            emsg = 'Image comparion would have failed: {}'
+                            print(emsg.format(msg))
+                        else:
+                            emsg = 'Image comparison failed: {}'
+                            raise ValueError(emsg.format(msg))
 
             if _DISPLAY_FIGURES:
-                if err:
-                    print('Image comparison would have failed. Message: %s' % err)
                 plt.show()
-            else:
-                assert not err, 'Image comparison failed. Message: %s' % err
 
         finally:
             plt.close()
@@ -755,13 +839,31 @@ class IrisTest(unittest.TestCase):
         # Return patch replacement object.
         return start_result
 
+    def assertArrayShapeStats(self, result, shape, mean, std_dev):
+        """
+        Assert that the result, a cube, has the provided shape and that the
+        mean and standard deviation of the data array are also as provided.
+        Thus build confidence that a cube processing operation, such as a
+        cube.regrid, has maintained its behaviour.
+
+        """
+        self.assertEqual(result.shape, shape)
+        self.assertAlmostEqual(result.data.mean(), mean, places=5)
+        self.assertAlmostEqual(result.data.std(), std_dev, places=5)
+
 
 get_result_path = IrisTest.get_result_path
 
 
 class GraphicsTest(IrisTest):
 
+    # nose directive: dispatch tests concurrently.
+    _multiprocess_can_split_ = True
+
     def setUp(self):
+        # Acquire threading non re-entrant blocking lock to ensure
+        # thread-safe plotting.
+        _lock.acquire()
         # Make sure we have no unclosed plots from previous tests before
         # generating this one.
         if MPL_AVAILABLE:
@@ -772,10 +874,11 @@ class GraphicsTest(IrisTest):
         # in an odd state, so we make sure it's been disposed of.
         if MPL_AVAILABLE:
             plt.close('all')
+        # Release the non re-entrant blocking lock.
+        _lock.release()
 
 
 class TestGribMessage(IrisTest):
-
     def assertGribMessageContents(self, filename, contents):
         """
         Evaluate whether all messages in a GRIB2 file contain the provided
@@ -944,6 +1047,11 @@ skip_nc_time_axis = unittest.skipIf(
     'Test(s) require "nc_time_axis", which is not available.')
 
 
+skip_inet = unittest.skipIf(not INET_AVAILABLE,
+                            ('Test(s) require an "internet connection", '
+                             'which is not available.'))
+
+
 def no_warnings(func):
     """
     Provides a decorator to ensure that there are no warnings raised
